@@ -6,7 +6,6 @@ from __future__ import division
 import numpy as np
 import numpy.polynomial.chebyshev as ch
 from numpy.testing import *
-from exceptions import TypeError, ValueError
 
 def trim(x) :
     return ch.chebtrim(x, tol=1e-6)
@@ -79,6 +78,14 @@ class TestArithmetic(TestCase) :
                 res = ch.chebsub([0]*i + [1], [0]*j + [1])
                 assert_equal(trim(res), trim(tgt), err_msg=msg)
 
+    def test_chebmulx(self):
+        assert_equal(ch.chebmulx([0]), [0])
+        assert_equal(ch.chebmulx([1]), [0,1])
+        for i in range(1, 5):
+            ser = [0]*i + [1]
+            tgt = [0]*(i - 1) + [.5, 0, .5]
+            assert_equal(ch.chebmulx(ser), tgt)
+
     def test_chebmul(self) :
         for i in range(5) :
             for j in range(5) :
@@ -105,7 +112,7 @@ class TestArithmetic(TestCase) :
             return x*(x**2 - 1)
 
         #check empty input
-        assert_equal(ch.chebval([], 1).size, 0)
+        assert_equal(ch.chebval([], [1]).size, 0)
 
         #check normal input)
         for i in range(5) :
@@ -137,10 +144,15 @@ class TestCalculus(TestCase) :
 
     def test_chebint(self) :
         # check exceptions
+        assert_raises(ValueError, ch.chebint, [0], .5)
         assert_raises(ValueError, ch.chebint, [0], -1)
         assert_raises(ValueError, ch.chebint, [0], 1, [0,0])
-        assert_raises(ValueError, ch.chebint, [0], 1, lbnd=[0,0])
-        assert_raises(ValueError, ch.chebint, [0], 1, scl=[0,0])
+
+        # test integration of zero polynomial
+        for i in range(2, 5):
+            k = [0]*(i - 2) + [1]
+            res = ch.chebint([0], m=i, k=k)
+            assert_almost_equal(res, [0, 1])
 
         # check single integration with integration constant
         for i in range(5) :
@@ -212,18 +224,22 @@ class TestCalculus(TestCase) :
 
     def test_chebder(self) :
         # check exceptions
+        assert_raises(ValueError, ch.chebder, [0], .5)
         assert_raises(ValueError, ch.chebder, [0], -1)
+
         # check that zeroth deriviative does nothing
         for i in range(5) :
             tgt = [1] + [0]*i
             res = ch.chebder(tgt, m=0)
             assert_equal(trim(res), trim(tgt))
+
         # check that derivation is the inverse of integration
         for i in range(5) :
             for j in range(2,5) :
                 tgt = [1] + [0]*i
                 res = ch.chebder(ch.chebint(tgt, m=j), m=j)
                 assert_almost_equal(trim(res), trim(tgt))
+
         # check derivation with scaling
         for i in range(5) :
             for j in range(2,5) :
@@ -259,6 +275,7 @@ class TestMisc(TestCase) :
         for i in range(4) :
             coef = [0]*i + [1]
             assert_almost_equal(v[...,i], ch.chebval(x, coef))
+
         # check for 2d x
         x = np.array([[1,2],[3,4],[5,6]])
         v = ch.chebvander(x, 3)
@@ -270,6 +287,7 @@ class TestMisc(TestCase) :
     def test_chebfit(self) :
         def f(x) :
             return x*(x - 1)*(x - 2)
+
         # Test exceptions
         assert_raises(ValueError, ch.chebfit, [1],    [1],     -1)
         assert_raises(TypeError,  ch.chebfit, [[1]],  [1],      0)
@@ -277,22 +295,40 @@ class TestMisc(TestCase) :
         assert_raises(TypeError,  ch.chebfit, [1],    [[[1]]],  0)
         assert_raises(TypeError,  ch.chebfit, [1, 2], [1],      0)
         assert_raises(TypeError,  ch.chebfit, [1],    [1, 2],   0)
+        assert_raises(TypeError,  ch.chebfit, [1],    [1],   0, w=[[1]])
+        assert_raises(TypeError,  ch.chebfit, [1],    [1],   0, w=[1,1])
+
         # Test fit
         x = np.linspace(0,2)
         y = f(x)
-        coef = ch.chebfit(x, y, 3)
-        assert_equal(len(coef), 4)
-        assert_almost_equal(ch.chebval(x, coef), y)
-        coef = ch.chebfit(x, y, 4)
-        assert_equal(len(coef), 5)
-        assert_almost_equal(ch.chebval(x, coef), y)
-        coef2d = ch.chebfit(x, np.array([y,y]).T, 4)
-        assert_almost_equal(coef2d, np.array([coef,coef]).T)
+        #
+        coef3 = ch.chebfit(x, y, 3)
+        assert_equal(len(coef3), 4)
+        assert_almost_equal(ch.chebval(x, coef3), y)
+        #
+        coef4 = ch.chebfit(x, y, 4)
+        assert_equal(len(coef4), 5)
+        assert_almost_equal(ch.chebval(x, coef4), y)
+        #
+        coef2d = ch.chebfit(x, np.array([y,y]).T, 3)
+        assert_almost_equal(coef2d, np.array([coef3,coef3]).T)
+        # test weighting
+        w = np.zeros_like(x)
+        yw = y.copy()
+        w[1::2] = 1
+        y[0::2] = 0
+        wcoef3 = ch.chebfit(x, yw, 3, w=w)
+        assert_almost_equal(wcoef3, coef3)
+        #
+        wcoef2d = ch.chebfit(x, np.array([yw,yw]).T, 3, w=w)
+        assert_almost_equal(wcoef2d, np.array([coef3,coef3]).T)
 
     def test_chebtrim(self) :
         coef = [2, -1, 1, 0]
+
         # Test exceptions
         assert_raises(ValueError, ch.chebtrim, coef, -1)
+
         # Test results
         assert_equal(ch.chebtrim(coef), coef[:-1])
         assert_equal(ch.chebtrim(coef, 1), coef[:-3])
@@ -309,6 +345,39 @@ class TestMisc(TestCase) :
         for i in range(10) :
             assert_equal(ch.poly2cheb(Tlist[i]), [0]*i + [1])
 
+    def test_chebpts1(self):
+        #test exceptions
+        assert_raises(ValueError, ch.chebpts1, 1.5)
+        assert_raises(ValueError, ch.chebpts1, 0)
+
+        #test points
+        tgt = [0]
+        assert_almost_equal(ch.chebpts1(1), tgt)
+        tgt = [-0.70710678118654746, 0.70710678118654746]
+        assert_almost_equal(ch.chebpts1(2), tgt)
+        tgt = [-0.86602540378443871, 0, 0.86602540378443871]
+        assert_almost_equal(ch.chebpts1(3), tgt)
+        tgt = [-0.9238795325, -0.3826834323,  0.3826834323,  0.9238795325]
+        assert_almost_equal(ch.chebpts1(4), tgt)
+
+
+    def test_chebpts2(self):
+        #test exceptions
+        assert_raises(ValueError, ch.chebpts2, 1.5)
+        assert_raises(ValueError, ch.chebpts2, 1)
+
+        #test points
+        tgt = [-1, 1]
+        assert_almost_equal(ch.chebpts2(2), tgt)
+        tgt = [-1, 0, 1]
+        assert_almost_equal(ch.chebpts2(3), tgt)
+        tgt = [-1, -0.5, .5, 1]
+        assert_almost_equal(ch.chebpts2(4), tgt)
+        tgt = [-1.0, -0.707106781187, 0, 0.707106781187, 1.0]
+        assert_almost_equal(ch.chebpts2(5), tgt)
+
+
+
 
 class TestChebyshevClass(TestCase) :
 
@@ -390,6 +459,17 @@ class TestChebyshevClass(TestCase) :
         xx = 2*x - 1
         assert_almost_equal(self.p2(x), self.p1(xx))
 
+    def test_degree(self) :
+        assert_equal(self.p1.degree(), 2)
+
+    def test_trimdeg(self) :
+        assert_raises(ValueError, self.p1.cutdeg, .5)
+        assert_raises(ValueError, self.p1.cutdeg, -1)
+        assert_equal(len(self.p1.cutdeg(3)), 3)
+        assert_equal(len(self.p1.cutdeg(2)), 3)
+        assert_equal(len(self.p1.cutdeg(1)), 2)
+        assert_equal(len(self.p1.cutdeg(0)), 1)
+
     def test_convert(self) :
         x = np.linspace(-1,1)
         p = self.p1.convert(domain=[0,1])
@@ -407,6 +487,7 @@ class TestChebyshevClass(TestCase) :
         assert_equal(p.trim(1e-5).coef, coef[:1])
 
     def test_truncate(self) :
+        assert_raises(ValueError, self.p1.truncate, .5)
         assert_raises(ValueError, self.p1.truncate, 0)
         assert_equal(len(self.p1.truncate(4)), 3)
         assert_equal(len(self.p1.truncate(3)), 3)
@@ -420,6 +501,8 @@ class TestChebyshevClass(TestCase) :
     def test_integ(self) :
         p = self.p2.integ()
         assert_almost_equal(p.coef, ch.chebint([1,2,3], 1, 0, scl=.5))
+        p = self.p2.integ(lbnd=0)
+        assert_almost_equal(p(0), 0)
         p = self.p2.integ(1, 1)
         assert_almost_equal(p.coef, ch.chebint([1,2,3], 1, 1, scl=.5))
         p = self.p2.integ(2, [1, 2])
@@ -436,6 +519,13 @@ class TestChebyshevClass(TestCase) :
         tgt = [0, .5, 1]
         assert_almost_equal(res, tgt)
 
+    def test_linspace(self):
+        xdes = np.linspace(0, 1, 20)
+        ydes = self.p2(xdes)
+        xres, yres = self.p2.linspace(20)
+        assert_almost_equal(xres, xdes)
+        assert_almost_equal(yres, ydes)
+
     def test_fromroots(self) :
         roots = [0, .5, 1]
         p = ch.Chebyshev.fromroots(roots, domain=[0, 1])
@@ -448,11 +538,25 @@ class TestChebyshevClass(TestCase) :
             return x*(x - 1)*(x - 2)
         x = np.linspace(0,3)
         y = f(x)
+
+        # test default value of domain
         p = ch.Chebyshev.fit(x, y, 3)
-        assert_almost_equal(p(x), y)
+        assert_almost_equal(p.domain, [0,3])
+
+        # test that fit works in given domains
         p = ch.Chebyshev.fit(x, y, 3, None)
         assert_almost_equal(p(x), y)
         assert_almost_equal(p.domain, [0,3])
+        p = ch.Chebyshev.fit(x, y, 3, [])
+        assert_almost_equal(p(x), y)
+        assert_almost_equal(p.domain, [-1, 1])
+        # test that fit accepts weights.
+        w = np.zeros_like(x)
+        yw = y.copy()
+        w[1::2] = 1
+        yw[0::2] = 0
+        p = ch.Chebyshev.fit(x, yw, 3, w=w)
+        assert_almost_equal(p(x), y)
 
     def test_identity(self) :
         x = np.linspace(0,3)
